@@ -94,6 +94,8 @@ class ConnectionLoader(ParallelLoader.ParallelLoader):
 
         super().__init__(db, dry_run=dry_run)
 
+        self.type = "connection"
+
     def generate_batch(self, entity_data):
 
         q = []
@@ -148,16 +150,3 @@ class ConnectionLoader(ParallelLoader.ParallelLoader):
             print(q)
 
         return q, []
-
-    def print_stats(self):
-
-        print("====== ApertureDB Connection Loader Stats ======")
-
-        times = np.array(self.times_arr)
-        print("Avg Query time(s):", np.mean(times))
-        print("Query time std:", np.std (times))
-        print("Avg Query Throughput (connections/s)):",
-            1 / np.mean(times) * self.batchsize * self.numthreads)
-
-        print("Total time(s):", self.ingestion_time)
-        print("================================================")
